@@ -38,8 +38,9 @@ import java.io.Writer;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 import static org.hamcrest.CoreMatchers.is;
-import static org.junit.Assert.assertThat;
+import static org.hamcrest.MatcherAssert.assertThat;
 
+@SuppressWarnings( "deprecation" )
 public class IOUtilTest
 {
 
